/* This file is a modified version of Daniel Mazzoni's modified version of "cylinder2D". Just to make things very clear;)
 Anders Bakke Krogvig
 "Flow acoutics in flow pipes with a small variable length cavity"
 Master thesis, acoustics
 NTNU, spring 2012
 
 */

/* This file is part of the Palabos library.
 * Copyright (C) 2009, 2010 Jonas Latt
 * E-mail contact: jonas@lbmethod.org
 * The most recent release of Palabos can be downloaded at
 * <http://www.lbmethod.org/palabos/>
 *
 * The library Palabos is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * The library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/** file
 * Flow through a 2D pipe having a cavity close to the inlet.
 *This example makes use of bounce-back nodes to
 * describe the shape of the rounded pipe and cavity edges.
 * The outlet is modeled through a
 * Neumann (zero velocity-gradient) condition.
 * Modeled by Daniel Mazzoni may/june 2011.
 */

#include "palabos2D.h"
#include "palabos2D.hh"
#include <vector>
#include <cmath>
#include <iostream>
#include <fstream>

#include <string>

using namespace plb;
using namespace plb::descriptors;
using namespace std;

typedef double T;
const T pi = (T)4.0*atan(1.0) ;
plint iT;

#define DESCRIPTOR D2Q9Descriptor

// Pulse of the flow velocity
T pulsation(T iT, T amplitude, T frequence, IncomprFlowParam<T> const& parameters) {
    T uMax = parameters.getLatticeU() ;
    T t    = iT*parameters.getDeltaT();
	return amplitude * cos(2 * pi* frequence * t) ;
}

//Turbulent flow in a circular tube with smooth walls
// http://ielnx1.epfl.ch/e-lin/Ryhming/documents/chapters/documents_published/doc6/node226.html
// Vx = ( y/a )^(1/nn) nn =  6 si Re =     4 000 ( a = rayon du tube )
//                     nn =  7 si Re =   100 000
//                     nn = 10 si Re = 2 000 000
T turbulentFlowVelicity(plint composante,plint iX, plint iY, IncomprFlowParam<T> const& parameters,
		                plint xTub, plint yTub, plint xCav, plint lCav, T amplitude, T frequence) {
    plint nx = parameters.getNx();
    plint ny = parameters.getNy();
    T nn = 6.;        // Profile for exhibit turbulent
    T mm = 6.;        // Exponent for the connection between the input section and the tube
    T x0 = 0.;        // Abscissa upstream connection speed for the exponential
	T x1 = (T)xTub;   // Abscissa downstream connection speed for exponential
    T y0 = 0.;        // Ordinate upstream connection speed for exponential
    T y1 = (T)yTub;   // Y-connection downstream of the exponential for speed
	T y2 = (T)(ny-1-yTub); // Upper limit. of flow in the tube
	T y3 = (T)(ny-1);      // Upper limit. of flow in the tube
    T xx = (T)iX;
    T yy = (T)iY;
    T yx = (xx<x1) ? y0+(y1-y0)/2.*(1.+sin(pi*(pow((xx-x0),mm)/pow((x1-x0),mm)-0.5))) : y1;
	                //Particle trajectory at the entrance of the tube
    T yp = (xx<x1) ? (y1-y0)/2. * mm * pi * pow(xx-x0,mm-1.) / pow(x1-x0,mm)
		 * cos(pi*(pow(xx-x0,mm)/pow(x1-x0,mm)-0.5)) : 0.; // Derivative of the trajectory
	T rs = (y2-y1)/(y3-y0-2.*yx) ; // rs<=1 = Rpport (Sections tube) / (Section a l'ascisse x) = S(x1) / S(x)
	T vx = rs * parameters.getLatticeU() ; // U(x) = uMax * ( S(x1) / S(x) )
	T vy = yp * parameters.getLatticeU() * (y3+y0-2.*yy)/(y3-y0); // vy change de signe entre y0 et y3, vy changes sign between
	T yc = (yy<((yx+(y3-yx))/2.)) ? yy-yx : (y3-yx)-yy;
	T up = ((yy>yx)&&(yy<(y3-yx))) ? pow(2.*yc/((y3-yx)-yx),(1./nn)) : 0; // Boundary layer profile
	T vitesse = (composante==0) ? vx : vy ;

	// pcout<<"  6  "<<endl; // ZZZZZ
	// pcout<<" xTub = "<<xTub<<" yTub = "<<yTub<<" xCav = "<<xCav<<" lCav = "<<lCav<< endl;
	// std::string sinputString;
	// std::cin >> sinputString;

    T t = iT * parameters.getDeltaT();

    return up * vitesse;
    // return up * (vitesse + 0.5*sin(8*atan(1)*34.56*t)) ;  // 1
    //return 0.5*sin(8*atan(1)*34.56*t);                     // 2
    //return up * vitesse * (1. + 0.5*sin(8*atan(1)*34.56*t)) ;  //test win1
    //return up * vitesse *(1. + 10*sin(8*atan(1)*34.56*t));  // 4 skal kjoresi morgen.
    //return 4.*(1.+0.5*sin(two_pi*99.76*t)) * parameters.getLatticeU();
    //return 0.5*sin(8*atan(1)*34.56*t) * parameters.getLatticeU(); //modepulsTest
}

// Linearly decreasing pressure profile
T turbulentFlowPressure(plint iX, IncomprFlowParam<T> const& parameters) {
    T Lx = parameters.getNx()-1;
    T Ly = parameters.getNy()-1;
    return 8.*parameters.getLatticeNu()*parameters.getLatticeU() / (Ly*Ly) * (Lx/(T)2-(T)iX);
}

// Convert pressure to density according to ideal gas law
T turbulentFlowDensity(plint iX, IncomprFlowParam<T> const& parameters) {
    return turbulentFlowPressure(iX,parameters)*DESCRIPTOR<T>::invCs2 + (T)1;
}

// A functional, used to initialize the velocity for the boundary conditions
template<typename T>
class TurbulentFlowVelocity {
public:
    TurbulentFlowVelocity(IncomprFlowParam<T> parameters_,
			plint xTub_, plint yTub_, plint xCav_, plint lCav_, T amplitude_, T frequence_)
        : parameters(parameters_), xTub(xTub_), yTub(yTub_), xCav(xCav_), lCav(lCav_),
		amplitude(amplitude_), frequence(frequence_)
    { }
    void operator()(plint iX, plint iY, Array<T,2>& u) const {
        u[0] = turbulentFlowVelicity(0, iX, iY, parameters, xTub, yTub, xCav, lCav, amplitude, frequence);
        u[1] = turbulentFlowVelicity(1, iX, iY, parameters, xTub, yTub, xCav, lCav, amplitude, frequence);
    }
private:
    IncomprFlowParam<T> parameters;
	T amplitude, frequence;
	plint xTub, yTub, xCav, lCav;
};

// A functional, used to initialize a pressure boundary to constant density
template<typename T>
class ConstantDensity {
public:
    ConstantDensity(T density_)
        : density(density_)
    { }
    T operator()(plint iX, plint iY) const {
        return density;
    }
private:
    T density;
};

// A functional, used to create an initial condition for the density and velocity
template<typename T>
class TurbulentFlowVelocityAndDensity {
public:
    TurbulentFlowVelocityAndDensity(IncomprFlowParam<T> parameters_,
			plint xTub_, plint yTub_, plint xCav_, plint lCav_, T amplitude_, T frequence_)
        : parameters(parameters_), xTub(xTub_), yTub(yTub_), xCav(xCav_), lCav(lCav_),
		amplitude(amplitude_), frequence(frequence_)
	{ }
    void operator()(plint iX, plint iY, T& rho, Array<T,2>& u) const {
        rho  = turbulentFlowDensity(iX,parameters);
        u[0] = turbulentFlowVelicity(0, iX, iY, parameters, xTub, yTub, xCav, lCav, amplitude, frequence);
        u[1] = turbulentFlowVelicity(1, iX, iY, parameters, xTub, yTub, xCav, lCav, amplitude, frequence);
    }
private:
    IncomprFlowParam<T> parameters;
	T amplitude, frequence;
	plint xTub, yTub, xCav, lCav;
};

// A functional, used to instantiate bounce-back nodes at the locations of the cylinder
template<typename T>
class CylinderShapeDomain2D : public plb::DomainFunctional2D {
public:
    CylinderShapeDomain2D(plb::plint cx_, plb::plint cy_, plb::plint radius)
        : cx(cx_),
          cy(cy_),
          radiusSqr(plb::util::sqr(radius))
    { }
    virtual bool operator() (plb::plint iX, plb::plint iY) const {
        return plb::util::sqr(iX-cx) + plb::util::sqr(iY-cy) <= radiusSqr;
    }
    virtual CylinderShapeDomain2D<T>* clone() const {
        return new CylinderShapeDomain2D<T>(*this);
    }
private:
    plb::plint cx;
    plb::plint cy;
    plb::plint radiusSqr;
};

// A functional, used to instantiate bounce-back nodes at the locations of a rectangle
template<typename T>
class RectangleShapeDomain2D : public plb::DomainFunctional2D {
public:
    RectangleShapeDomain2D(plb::plint x0_, plb::plint x1_, plb::plint y0_, plb::plint y1_)
        : x0(x0_),
          x1(x1_),
          y0(y0_),
          y1(y1_)
    { }
    virtual bool operator() (plb::plint iX, plb::plint iY) const {
		T dif1 = x0-iX;
		T dif2 = x1-iX;
		T dif3 = y0-iY;
		T dif4 = y1-iY;
		return (dif1 < 0)&&(dif2 > 0)&&(dif3 < 0)&&(dif4 > 0);
    }
    virtual RectangleShapeDomain2D<T>* clone() const {
        return new RectangleShapeDomain2D<T>(*this);
    }
private:
    plb::plint x0;
    plb::plint x1;
    plb::plint y0;
    plb::plint y1;
};

void boundaryConditionsSetup( MultiBlockLattice2D<T,DESCRIPTOR>& lattice,
                              IncomprFlowParam<T> const& parameters,
                              OnLatticeBoundaryCondition2D<T,DESCRIPTOR>& boundaryCondition,
							  plint xTub, plint yTub, plint xCav, plint lCav, T amplitude, T frequence)
{
    const plint nx = parameters.getNx();
    const plint ny = parameters.getNy();

    plint x0 = 0;
    plint x1 = nx-1;

    plint xtub = xTub; // Upstream edge of the cavity
    plint xcav = xCav; // Upstream edge of the cavity
    plint lcav = lCav; // Length of the cavity

    plint rayon1 = 10;  // Rounded edges of cavity
    plint demiR2 = yTub/4;
    plint rayon2 = 2*demiR2;

    plint y0 = 0;
    plint y1 = yTub;
	plint y2 = ny-1-yTub;
	plint y3 = ny-1;

    //pcout<<"  4  "<<endl; // ZZZZZ
	// pcout<<" xTub = "<<xTub<<" yTub = "<<yTub<<" xCav = "<<xCav<<" lCav = "<<lCav<< endl;
	// std::string sinputString;
	// std::cin >> sinputString;

    // The Neumann condition at the output speed
    Box2D outlet(nx-1, nx-1, ny/2+1, ny-2);

    // Create Velocity boundary conditions everywhere
    boundaryCondition.setVelocityConditionOnBlockBoundaries (
            lattice, Box2D(0, 0, 1, ny-2) );
    boundaryCondition.setVelocityConditionOnBlockBoundaries (
            lattice, Box2D(0, nx-1, 0, 0) );
    boundaryCondition.setVelocityConditionOnBlockBoundaries (
            lattice, Box2D(0, nx-1, ny-1, ny-1) );

    // .. except on right boundary, where we prefer an outflow condition
    //    (zero velocity-gradient).
    boundaryCondition.setVelocityConditionOnBlockBoundaries (
            lattice, Box2D(nx-1, nx-1, 1, ny-2), boundary::outflow );

	// pcout<<"  5  "<<endl; // ZZZZZZ
	// pcout<<" xTub = "<<xTub<<" yTub = "<<yTub<<" xCav = "<<xCav<<" lCav = "<<lCav<< endl;
	// std::cin >> sinputString;

    // The condition of speed limits in speed is given by TurbulentFlowVelocity
    setBoundaryVelocity (
            lattice, lattice.getBoundingBox(),
            TurbulentFlowVelocity<T>(parameters, xTub, yTub, xCav, lCav, amplitude, frequence) );

	// pcout<<"  7  "<<endl; // ZZZZZZ
	// pcout<<" xTub = "<<xTub<<" yTub = "<<yTub<<" xCav = "<<xCav<<" lCav = "<<lCav<< endl;
	// std::cin >> sinputString;

    setBoundaryDensity (
            lattice, outlet,
            ConstantDensity<T>(1.) );

	// pcout<<"  8  "<<endl; // ZZZZZZ
	// pcout<<" xTub = "<<xTub<<" yTub = "<<yTub<<" xCav = "<<xCav<<" lCav = "<<lCav<< endl;
	// std::cin >> sinputString;

    initializeAtEquilibrium (
            lattice, lattice.getBoundingBox(),
            TurbulentFlowVelocityAndDensity<T>(parameters, xTub, yTub, xCav, lCav, amplitude, frequence) );

	// pcout<<"  9  "<<endl; // ZZZZZZ
	// pcout<<" xTub = "<<xTub<<" yTub = "<<yTub<<" xCav = "<<xCav<<" lCav = "<<lCav<< endl;
	// std::cin >> sinputString;

	//consturction of the geometry

    //Definingthe tube intake with rounded edge (1) and tube (2):
    // (1) We define the rounded upstream of the tub:
    // Using radius rayon 2:
    defineDynamics(lattice, lattice.getBoundingBox(),
                   new CylinderShapeDomain2D<T>(xtub+rayon1,y1-rayon1,rayon1),
                   new plb::BounceBack<T,DESCRIPTOR>);

    defineDynamics(lattice, lattice.getBoundingBox(),
                   new
				   RectangleShapeDomain2D<T>(xtub,xtub+rayon1,y0,y1-rayon1),
                   new plb::BounceBack<T,DESCRIPTOR>);

     // Using radius rayon 1:
    defineDynamics(lattice, lattice.getBoundingBox(),
                   new CylinderShapeDomain2D<T>(xtub+rayon1,y2+rayon1,rayon1),
                   new plb::BounceBack<T,DESCRIPTOR>);

    defineDynamics(lattice, lattice.getBoundingBox(),
                   new RectangleShapeDomain2D<T>(xtub,xtub+rayon1,y2+rayon1,y3),
                   new plb::BounceBack<T,DESCRIPTOR>);

    // (2) We define the portion of tube upstream of the cavity
    defineDynamics(lattice, lattice.getBoundingBox(),
                   new RectangleShapeDomain2D<T>(xtub+rayon1,xcav-rayon1,y0,y1),
                   new plb::BounceBack<T,DESCRIPTOR>);

    defineDynamics(lattice, lattice.getBoundingBox(),
                   new RectangleShapeDomain2D<T>(xtub+rayon1,xcav-rayon1,y2,y3),
                   new plb::BounceBack<T,DESCRIPTOR>);

/////////////////////////////

    // Defining the cavity:
    // We define the rounded upstream of the cavity
    defineDynamics(lattice, lattice.getBoundingBox(),
                   new CylinderShapeDomain2D<T>(xcav-rayon1,y1-rayon1,rayon1),
                   new plb::BounceBack<T,DESCRIPTOR>);

    defineDynamics(lattice, lattice.getBoundingBox(),
                   // new RectangleShapeDomain2D<T>(xcav-2*rayon1,xcav,y0,y1-rayon1),
                   new RectangleShapeDomain2D<T>(xcav-rayon1,xcav,y0,y1-rayon1),  // why xcav-2*rayon1 and not xcav-rayon1?
                   new plb::BounceBack<T,DESCRIPTOR>);

    defineDynamics(lattice, lattice.getBoundingBox(),
                   new CylinderShapeDomain2D<T>(xcav-rayon1,y2+rayon1,rayon1),
                   new plb::BounceBack<T,DESCRIPTOR>);

    defineDynamics(lattice, lattice.getBoundingBox(),
                   //new RectangleShapeDomain2D<T>(xcav-2*rayon1,xcav,y2+rayon1,y3),
                   new RectangleShapeDomain2D<T>(xcav-rayon1,xcav,y2+rayon1,y3),
                   new plb::BounceBack<T,DESCRIPTOR>);
                        // why xcav-2*rayon1 and not xcav-rayon1?

    //Defining the cavity space:
    //Already done by BC


    // We define the rounding down of the cavity
    defineDynamics(lattice, lattice.getBoundingBox(),
                   new CylinderShapeDomain2D<T>(xcav+lcav+rayon1,y1-rayon1,rayon1),
                   new plb::BounceBack<T,DESCRIPTOR>);

    defineDynamics(lattice, lattice.getBoundingBox(),
                   new RectangleShapeDomain2D<T>(xcav+lcav,xcav+lcav+2*rayon1,y0,y1-rayon1),
                   new plb::BounceBack<T,DESCRIPTOR>);

    defineDynamics(lattice, lattice.getBoundingBox(),
                   new CylinderShapeDomain2D<T>(xcav+lcav+rayon1,y2+rayon1,rayon1),
                   new plb::BounceBack<T,DESCRIPTOR>);

    defineDynamics(lattice, lattice.getBoundingBox(),
                   new RectangleShapeDomain2D<T>(xcav+lcav,xcav+lcav+2*rayon1,y2+rayon1,y3),
                   new plb::BounceBack<T,DESCRIPTOR>);

    // We define the tube portion downstream of the cavity
    defineDynamics(lattice, lattice.getBoundingBox(),
                   new RectangleShapeDomain2D<T>(xcav+lcav+rayon1,x1,y0,y1),
                   new plb::BounceBack<T,DESCRIPTOR>);

    defineDynamics(lattice, lattice.getBoundingBox(),
                   new RectangleShapeDomain2D<T>(xcav+lcav+rayon1,x1,y2,y3),
                   new plb::BounceBack<T,DESCRIPTOR>);

	// For a single tube :
    // defineDynamics(lattice, lattice.getBoundingBox(),
    //                new RectangleShapeDomain2D<T>(x0,x1,y0,y1),
    //                new plb::BounceBack<T,DESCRIPTOR>);

	// pcout<<"  10  "<<endl; // ZZZZZZ
	// pcout<<" xTub = "<<xTub<<" yTub = "<<yTub<<" xCav = "<<xCav<<" lCav = "<<lCav<< endl;
	// std::cin >> sinputString;

    lattice.initialize();
}

void writeGif(MultiBlockLattice2D<T,DESCRIPTOR>& lattice, plint iter)
{
    const plint imSize = 800;
    ImageWriter<T> imageWriter("leeloo");
    imageWriter.writeScaledGif(createFileName("u", iter, 6),
                               *computeVelocityNorm(lattice),
                               imSize, imSize );
}

void writeVTK(MultiBlockLattice2D<T,DESCRIPTOR>& lattice,
              IncomprFlowParam<T> const& parameters, plint iter)
{
    T dx = parameters.getDeltaX();
    T dt = parameters.getDeltaT();
    VtkImageOutput2D<T> vtkOut(createFileName("vtk", iter, 6), dx);
    vtkOut.writeData<float>(*computeVelocityNorm(lattice), "velocityNorm", dx/dt);
    vtkOut.writeData<2,float>(*computeVelocity(lattice), "velocity", dx/dt);
}

/* Definition de la geometrie

+===========+      +=============+-------------------------y3
            |      |             |
            |      |             |
            o======o             o======================+--y2


----> U


            o======o             o======================+--y1
            |      |             |
            |      |             |
+===========+      +=============+-------------------------y0=0
|           |      |             |                      |
x0=0      xTub   xCav       xCav+lCav               x1=nx-1
*/
int main(int argc, char* argv[]) {
    plbInit(&argc, &argv);

    IncomprFlowParam<T> parameters(
            (T) 0.017, // c=1/sqrt(3) => u_LB = u_SI/sqrt(3)/340
            // (T) 5800., // Re = for a tube diameter of 1 cm. (Reynolds Number)
            (T) 5800, // Re = for a tube diameter of 1 cm. (Reynolds Number)
            500,      // N, Resolution
            4.,      //  length
            1.      // heigth
    );
    const plint nx = parameters.getNx();
    const plint ny = parameters.getNy();


    const plint rayon1 = 10;     // Rounded edges of cavity

    const plint y0 = 0;
	const plint y3 = ny-1;
    const plint y1 = ny/4;
	const plint y2 = y3-y1;

    const plint xTub = ny;
    const plint yTub = ny/4;
    const plint xCav = xTub+ny/2;  // Inlet length
    const plint lCav = ny/4 ; // Cavity length
    
    //pcout << "xTub: "  << xTub << endl;
    //pcout << "xCav: "  << xCav << endl;
    //pcout << "lCav: "  << lCav << endl;
 
    const T maxT        = (T)43.1;    // length of simulation in seconds
    const T logT        = (T)0.01;    
    const T imSave      = (T)1.0;   
    const T vtkSave     = (T)2.;     
    const T profileSave = (T).001;      
    const T initPointStream = (T)8.0;  // data recording delay
    T amplitude = 1.;
    T frequence = 1.;

    // Measuring points:
    const plint yM = (ny-1)/2;  //Midpoint in y-axis
    const plint x1 = nx-1;
    const plint x0 = 0;
    plint xData;
    const plint nPoint = 50;

	// Output directory
    global::directories().setOutputDir("./tmp/");
    // The following command opens a text-file, in which
    // the velocity-profile is written at successive time steps.
    //plb_ofstream Vx1Profiles("tmp/Vx1Profiles.dat");
    //plb_ofstream Vy1Profiles("tmp/Vy1Profiles.dat");
    //plb_ofstream Vx2Profiles("tmp/Vx2Profiles.dat");
    //plb_ofstream Vy2Profiles("tmp/Vy2Profiles.dat");
    plb_ofstream xAxisVxProfiles("tmp/xAxisVxProfiles.dat");
    //plb_ofstream pressureProfiles("tmp/xAxisPressureProfiles.dat");
    plb_ofstream densityProfiles("tmp/xAxisDensityProfiles.dat");
    plb_ofstream antennaVx("tmp/antennaVx.dat",ios::app);
    plb_ofstream antennaRho("tmp/antennaRho.dat",ios::app);

    // Compute only the x-direction of the velocity (direction of the flow).
    plint xComponent = 0;

    // Compute only the y-direction of the velocity (normal to the direction of the flow).
    plint yComponent = 1;

    writeLogFile(parameters, "Turbulent flow");

	// pcout << "  1  "<<endl; // ZZZZZZ
	// std::string sinputString;
	// std::cin >> sinputString;

    MultiBlockLattice2D<T, DESCRIPTOR> lattice (
            parameters.getNx(), parameters.getNy(),
            new RegularizedBGKdynamics<T,DESCRIPTOR>(parameters.getOmega()) );

	// pcout << "  2  "<<endl; // ZZZZZZ
	// std::cin >> sinputString;

    OnLatticeBoundaryCondition2D<T,DESCRIPTOR>*
        boundaryCondition = createLocalBoundaryCondition2D<T,DESCRIPTOR>();

	// pcout<<"  3  "<<endl; // ZZZZZZ
	// pcout<<" xTub = "<<xTub<<" yTub = "<<yTub<<" Cav = "<<xCav<<" lCav = "<<lCav<< endl;
	// std::cin >> sinputString;

    boundaryConditionsSetup(lattice, parameters, *boundaryCondition,
							xTub, yTub, xCav, lCav, amplitude, frequence);

    // Main loop over time iterations.
    for (plint iT=0; iT*parameters.getDeltaT()<maxT; ++iT) {
        // At this point, the state of the lattice corresponds to the
        //   discrete time iT. However, the stored averages (getStoredAverageEnergy
        //   and getStoredAverageDensity) correspond to the previous time iT-1.
     
        if (iT*parameters.getDeltaT() >= initPointStream){

       if (iT%parameters.nStep(imSave)==0) {
            pcout << "iT =  " << endl;
            pcout << "Saving Gif ..." << endl;
            writeGif(lattice, iT);
        }

        if (profileSave>0) {
            if (iT%parameters.nStep(profileSave)==0) {
             //   antennaT << (iT*dt) - initPointStream << endl;
                for (plint ix=1; ix<=nPoint; ix++) {
                    xData = (nx-1)/(nPoint)*(ix);
                    Box2D measurePoint(xData, xData, yM, yM);
                    antennaVx << setprecision(8) <<  *multiply ( parameters.getDeltaX()/parameters.getDeltaT(),
                            *computeVelocityComponent (lattice, measurePoint, xComponent) );
                }
                antennaVx << endl;
                for (plint ix=1; ix<=nPoint; ix++) {
                    xData = (nx-1)/(nPoint)*(ix);
                    Box2D measurePoint(xData, xData, yM, yM);
                    antennaRho << setprecision(8) <<  *computeDensity (lattice, measurePoint);
                }
                antennaRho << endl;
            }
        }

        // Trace Vx et Vy
        //if (iT%parameters.nStep(profileSave)==0) {

            // Measure in x dir along x-axis.
            // Definition of the section of track: in the middle of the tube, along the x-axis.
          //  Box2D profileSectionXaxis(x0, x1, yM, yM);
            //trace Vx along x-axis
           // xAxisVxProfiles
            //<< setprecision(4)
            // (2) Convert from lattice to physical units.
             //       << *multiply ( parameters.getDeltaX() / parameters.getDeltaT(),
                          // (1) Compute velocity norm along the chosen section.
               //           *computeVelocityComponent (lattice, profileSectionXaxis, xComponent) )
                //    << endl;

         //   densityProfiles
           //     << setprecision(6)
             //   << *computeDensity (lattice, profileSectionXaxis)
               // << endl;
        //}

        if (iT%parameters.nStep(logT)==0) {
            pcout << "step " << iT
                  << "; t=" << iT*parameters.getDeltaT()
                  << "; amplitude = " << amplitude
                  << "; frequence = " << frequence
                  << "; pulsation = " << pulsation((T)iT, amplitude, frequence, parameters)<<endl;
        }

        }
        // Lattice Boltzmann iteration step.
        lattice.collideAndStream();

        // At this point, the state of the lattice corresponds to the
        //   discrete time iT+1, and the stored averages are upgraded to time iT.
        if (iT%parameters.nStep(logT)==0) {
            pcout << "; av energy ="
                  << setprecision(10) << getStoredAverageEnergy<T>(lattice)
                  << "; av rho ="
                  << getStoredAverageDensity<T>(lattice) << endl;
        }
    }

    delete boundaryCondition;
}



